/**************************************************************/
/*                                                            */
/*   Program Name :  speana                                   */
/*                                                            */
/*      Version:  0.00  2002-05-20                            */
/*                1.41  2002-10-01                            */
/*                1.42  2002-10-11    18000->16000 bug fix    */
/*                1.43  2004-04-20    Sampling Frequency      */
/*                                      supported > 16MHz     */
/*                                                            */
/*      Copyright (c) 2002 T.Kondo/CRL All Right Reserved     */
/*                                                            */
/**************************************************************/
/*
 ho|ukah{[h擾f[^̃XyNg
        iItCj

          T.Kondo 2002.5.20
               V1.41 2002.10.1 g̃ftHgjAɕύX

    点 speana filename [mode [sekibun [pmode [comment [soffset]]]]]
          filename -- f[^t@C
                            0 Ƃƃt@C
                           tds.datag
               mode -- g[h
                       0: OXP[
                       1: jAXP[ iftHgj
                      -value ȑ֊֐\ivalue_gj
                      -1: ȑ֊֐tW
                      
               sekibun  -- ϕ(bPʁj
                       ftHg Pibj
               pmode -- vbg\foCX[h
                       0: XTERM  PostScriptt@CóiftHgj
                       1: PostScrip o͂̂ 
                       2: XTERM o͂̂ 
               comment  -- RgiOt㕔ɕ\jBXy[X܂܂ȂƁB
                           ȗꍇ͉b[h͂ɂȂ
                           FXy[X܂ރRg͉b[hœ͂邱ƁB
               soffset -- X^[gItZbgibj
*/
#include "chead.h"      // common header for IP-VLBI utilities

#include <cpgplot.h>
#include "libfft.h"
#include "libipvlbi.h"  // IP-VLBIp֐Q(by T.Kondo)
//char dev[] = "/dev/tds0";

/*  graphic display set */
#ifdef WIN32
#define PG_DISP "/GW"
#else
#define PG_DISP "/XTERM"
#endif
 
/* constatnts set */
#ifndef PIRA
#define PIRA  1.745329252E-2    // PI/180  degree to radian
#endif
#ifndef PIDE
#define PIDE  57.29577951       // 180/PI radian to degree
#endif
#ifndef TWOPI
#define TWOPI 6.283185307       // 2PI
#endif
#ifndef PI
#define PI    3.141592653
#endif

/*    Offset from dB to dBm conversion from A/D multibit sampling data */ 
#define OFFSET1  16.0     //1 bit sampling full power convert to 0 dB (theoretical)
      /* following constants are provisional  (measured on 2002/5/18) */
#define OFFSET2   9.0     //2 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET4  -1.0     //4 bit sampling data offset measured at 200kHz for PC(vlbi11)
#define OFFSET8 -26.0     //8 bit sampling data offset measured at 200kHz for PC(vlbi11)

typedef struct ip_vlbi_file
{
	unsigned char bit32[4];
	unsigned char bit64[8];
	//char buf[65536];
	int sfreq;
	int icnt;   //  current position number
       char* fname; //I[vĂt@C
       float fsample;  //TvOg(Hz)
	int   adbit;  // ADrbg
	long int seconds; // 00h00m00šoߕb
       int hh, mm, ss;
	int spatn; // sync pattern
	int syncok; // =1 for 32bit all 1 detection
	int format; // =0 for old format , =1 for new format
	FILE *stream;
       fpos_t pos;
	int fh;
	int ch;
	int numch;
	int ierr;
       double dtime;
       unsigned int bytesread;
} ip_vlbi_file_t;

int checkheader(ip_vlbi_file_t *S)
{
       /* Read 64bit Data header check to get sampling parameters */
       int iok, i;

       S->bytesread = fread( S->bit64, sizeof( char ), 8, S->stream );
       //printf("bytes read 1 %d\n",bytesread); //debug

       if( ferror( S->stream ) || S->bytesread==0 )
       {
           printf("checkheader: File EOF or Read Error!\n");
	       i=fclose( S->stream ) ;
		   return -1;
	   }


       headerchkn(S->bit64,&iok,&S->spatn,&S->adbit,&S->sfreq,&S->numch,
                        &S->seconds,&S->hh,&S->mm,&S->ss);
       if (iok == 0)   // sync detection
	   {
          printf("New Format (header 64bits) Sync Detected!!\n");
		  printf("File : %s\n",S->fname);
          printf("  A/D(bits) = %d  # of ch = %d  Sampling Freq(kHz) = %d\n",
                    S->adbit,S->numch, S->sfreq);
          printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",
                    S->hh,S->mm,S->ss,S->seconds);
          S->icnt=0;          
          return 0;
	   }
       else
       {
           printf("First 64bit is not header block!\n");
	       i=fclose( S->stream ) ;
           return -1;
       }
}

int fwdNsec(unsigned char* ibuf, int nsec, int numb, int imax,ip_vlbi_file_t *dat)
{
    int n, i;
    if(nsec <= 0) return 0;
	//printf("fwdNsec: nsec, numb, imax %d %d %d\n",nsec,numb,imax); //debug
    for(n=0; n<nsec; n++)
    {
        for(i=0; i< imax; i++)
        {
            dat->bytesread = fread( ibuf, sizeof( char ), numb, dat->stream );
            //printf("i, bytesread %d %d %d\n",i,dat->bytesread,ferror( dat->stream ));
            if( ferror( dat->stream ) || dat->bytesread==0 ){
		        return -1;
            }
		}
        if(checkheader(&(*dat)) < 0){
             return -1;
        }
     }
     return 0;
}

int fwdNsecHere(unsigned char* ibuf, int nsec, int numb, int imax,ip_vlbi_file_t *dat)
{
    int n, i;
    if(nsec <= 0) return 0;
	//printf("fwdNsec: nsec, numb, imax %d %d %d\n",nsec,numb,imax); //debug
    for(n=0; n<nsec; n++)
    {
        for(i=0; i< imax; i++)
        {
            dat->bytesread = fread( ibuf, sizeof( char ), numb, dat->stream );
            //printf("i, bytesread %d %d %d\n",i,dat->bytesread,ferror( dat->stream ));
            if( ferror( dat->stream ) || dat->bytesread==0 ){
		        return -1;
            }
		}
        /*
        if(checkheader(&(*dat)) < 0){
             return -1;
        }*/
     }
     return 0;
}


void plotspe_a(float *spow, int npos, int sfreq, int adbit, char *lab, char *lab2, int mode)
/*
        mode = 0; log frequency scale   1; linear frequency scale
*/
{
      //float cofst,fmaxkhz, fminkhz, fsampl, df, ymax, ymin, pmax, pmin, maxdbm, mindbm;
      float fmaxkhz, fminkhz, fsampl, df, ymax, ymin, pmax, pmin, maxdbm, mindbm;
      float *fdat, *powdat;
      //float x,y;
      char htitl[60], xtit[60], ytit[60];
      int n, npmax;

      strcpy(htitl,"POWER SPECTRUM");

      /* frequency axis generation */

      npmax=npos/2;

      fdat   = (float *)malloc(npmax*sizeof(float));
      powdat = (float *)malloc(npmax*sizeof(float));

      fsampl=(float)1000.0*(float)sfreq;
      df=fsampl/(float)npos;
      if(mode == 0) {    // log scale
         fmaxkhz=(float)log10(fsampl/2000.0);
         fminkhz=(float)log10(df/1000.0);
         for(n=0; n<npmax; n++){fdat[n]=(float)log10((float)n*df/1000.0); }
      } else {          //  linear scale
         fmaxkhz=(float)(fsampl/2000.0);
         fminkhz=0.0;
         for(n=0; n<npmax; n++){fdat[n]=(float)((float)n*df/1000.0); }
      }

      strcpy(xtit,"FREQUENCY (kHz)");
      strcpy(ytit,"POWER (dBm)");
      if( adbit==1 ){strcpy(ytit,"POWER (dB)"); }
      ymax=10.0;
      ymin=-80.0;
      if (adbit == 8 ){ ymin=-100; }    // extend dynamic range 

      /* data max, min check */
      for(n=0; n<npmax; n++)
      {
         powdat[n]=spow[n];
         if(n==0)
         {
            pmax=powdat[n];
            pmin=powdat[n];
         }
         else
         {
            if(powdat[n] > pmax){ pmax=powdat[n]; }
            if(powdat[n] < pmin){ pmin=powdat[n]; }
         }
      }

      maxdbm=pmax;
      mindbm=pmin;

      //goto exitend; //debug

      /* open graphics device */
      /*   move to main */
       //if (cpgopen("?") < 1)
      // if (cpgopen("/GF") < 1)
      // {
      //    goto exitend;
      // }

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      if(mode == 0){   
         cpgenv(fminkhz, fmaxkhz, ymin, ymax, 0, 10);   // log scale
      } else {         
         cpgenv(fminkhz, fmaxkhz, ymin, ymax, 0, 0);   // linear scale
      }
      
      /* label the axes */
       cpglab(xtit,ytit,htitl);

      /* sub label write*/
       cpgsch(1.5);
       cpgmtxt("T",1.5,0.5,0.5,lab);
       cpgmtxt("T",0.5,0.5,0.5,lab2);
       cpgsch(2.0);
      
      /* check data */
      if (pmax == pmin)
      {
          cpgmtxt("B",-6.0,0.5,0.5,"NO TIME VARIABLE");
          cpgmtxt("B",-4.0,0.5,0.5,"COMPONENT FOUND");
          /* label the axes */
          
      }
      else
      {


          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          //cpgslw(1);    // set line width
          cpgline(npmax,fdat,powdat); 

      }
      /* close the graphics device */
      /* move to main */
      //cpgclos();
  //exitend: 
     free(powdat);
     free(fdat);
}

void plotautocor(float *spow, int npos, int sfreq, int adbit, char *lab, char *lab2, int amode)
/*
        mode = 0; log frequency scale   1; linear frequency scale
*/
{
      //float cofst,tmax, tmin, fsampl, dt, ymax, ymin, pmax, pmin, maxdbm, mindbm;
      float tmax, tmin, fsampl, dt, ymax, ymin, pmax, pmin;
      float *tdat, *pdat;
      //float x,y;
      char htitl[60], xtit[60], ytit[60];
      int n, k, npmax, nhalf;

      strcpy(htitl,"AUTO CORRELATION (ABS)");

      /* time axis generation */

      if(amode <= 1) {
         npmax=npos;
      } else {
         npmax=amode;
      }
      tdat = (float *)malloc(npmax*sizeof(float));
      pdat = (float *)malloc(npmax*sizeof(float));
      nhalf=npmax/2;

      fsampl=(float)(1000.0*(float)sfreq);
      dt=(float)(1.0/fsampl);
      for(n=0; n<npmax; n++){
          tdat[n]=(float)((float)(n-nhalf)*dt*1.0e6);   // usec unit
      }
      tmax=tdat[npmax-1];
      tmin=tdat[0];

      strcpy(xtit,"DELAY TIME (us)");
      strcpy(ytit,"CORRELATION AMP");
      ymax=(float)1.1;
      ymin=0.0;

      /* data range limited copy and max, min check */
     
      for(n=0; n<npmax; n++)
      {
         k=n-nhalf;
         if(k < 0) k=npos+k;
         pdat[n]=spow[k];
         if(pdat[n]<0) pdat[n]=-pdat[n];  // to take absolute value
         if(n==0)
         {
            pmax=pdat[n];
            pmin=pdat[n];
         }
         else
         {
            if(pdat[n] > pmax){ pmax=pdat[n]; }
            if(pdat[n] < pmin){ pmin=pdat[n]; }
         }
      }
      

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      cpgenv(tmin, tmax, ymin, ymax, 0, 0);   // linear scale
      
      /* label the axes */
       cpglab(xtit,ytit,htitl);

      /* sub label write*/
       cpgsch(1.5);
       cpgmtxt("T",1.5,0.5,0.5,lab);
       cpgmtxt("T",0.5,0.5,0.5,lab2);
       cpgsch(2.0);
      /* check data */
      if (pmax == pmin)
      {
          cpgmtxt("B",-6.0,0.5,0.5,"NO DATA FOR PLOT");
          
      }
      else
      {
      
        /* plot the line graph */
         cpgline(npmax,tdat,pdat); 
      }

      /* move to main */
  //exitend: 
     free(pdat);
     free(tdat);
}


int main(int argc, char *argv[])
{
       //int spatn, sfreq, adbit, numch, kpos;
       int sfreq, adbit, kpos, numch;
       //int sfreqt,adbitt,numcht;
       int amode;    // >0 auto correlation plot mode 
       //long int seconds;
       //long int startsec,startsecdata;
       long int startoffset; // start time offset in sec
       //int hh, mm, ss;
       int nloop,sekibun;
       //int i,iok, imax, m, k, kaisu;
       int i,imax, m, k, kaisu;
       int smode,usampl,nfft,kplot;
	   //unsigned char bit64[8];
       unsigned char *ibuf;
	   int nbyte=5000;
       //FILE *stream;
       fpos_t pos;
       int bytesread;
       int bits, bytes, numb;
       //int mode=0;   // log frequency scale
       //int mode=1;   // linear frequency scale
       int mode, pmode; 
       int scount[4];
       float fsampl,tsampl,fact;
       int *idata;
       double *cdata;
       float *xdata, *ydata;
       double sum,dc;
       float pows, *spows;
       float pmax,pmin,pmaxdbm,pmindbm,cofst;
       char fname[60];
       //char timelab[30];
       char lab[60], lab2[60], iplab[40], ltime1st[60];
       //char cbuf[60];
       char labfsampl[10];
	//char outfile[] = "./tds.data";
	char* dflt_fname="tds.data";
       ip_vlbi_file_t X;

	if(argc == 1){
          printf("speana Ver1.43 (2004-04-20)\n");
          printf("Usage: speana filename [mode [sekibun [pmode [comment [soffset]]]]]\n");
          printf("\n");
          printf("         where  filename ---- Data file name\n");
          printf("                              if 0 is given, tds.data will be used\n");
          printf("                mode -------- Frequenxy axis mode\n");
          printf("                              =0 : log scale\n");
          printf("                              =1 : linear scale (default)\n");
          printf("                              =-1 : Auto correlation function plot\n");
          printf("                                     instead of spectrum\n");
          printf("                              =-N : N points auto correlation function plot\n");
          printf("                sekibun --- Averaging period (integer seconds)\n");
          printf("                             default is 1\n");
          printf("                pmode ---- Plot device selection\n");
          printf("                           =0 :  XTERM and PostScript file (pgplot.ps) out\n");
          printf("                           =1 :  PostScript file (pgplot.ps) out only\n");
          printf("                           =2 :  XTERM only\n");
          printf("                          default is 0\n");
          printf("                comment --- Comment appeared on a graph\n");
          printf("                            if omitted, you will be asked later.\n");
          printf("                soffset --- Start time offset in sec (integer seconds)\n");
          printf("                             default is 0\n");          exit(0);
	}

       strcpy(fname,*(argv+1));
	if (fname[0]=='0'){
          strcpy(fname,dflt_fname);
	}
       X.fname=fname;
	if (argc>=3)
	{
	   mode = atoi(*(argv + 2 ));
          if(mode <0 ){
            amode=-mode;
            mode=0;
          } else {
            amode=0;
          }
	}
       else
       {
          mode=1;     // default is linear frequency scale mode
          amode=0;
       }

	if (argc>=4)
	{
	   sekibun = atoi(*(argv + 3 ));
	}
       else
       {
          sekibun=1;     // default integration time (sec)
       }

	if (argc>=5)
	{
	   pmode = atoi(*(argv + 4 ));
	}
       else
       {
          pmode=0;     // default plot mode
       }


       printf("Data File is %s\n",fname);
       if( (X.stream  = fopen( X.fname, "rb" )) == NULL )
       {
         //perror( "̓t@CJ܂" );
         //exit( 1 );
	  // return -1;
         printf("File Open Error! \n");
		goto exitend2;
        }

        /* get comment */

	if (argc>=6){
          strcpy(iplab,*(argv+5));
       } else {
          /* enter board ID */
          /*
          printf("Enter IP-VLBI BOARD ID -> ");
          read1line(iplab);      // 1 line read from stdin
          printf("Board ID is %s\n",iplab);      
          strcpy(lab2,"IP-VLBI Board ID : ");
          strcat(lab2,iplab);
          */

          /* enter any comment */

          printf("Enter any comment -> ");
          read1line(iplab);      // 1 line read from stdin
          //printf("Board ID is %s\n",iplab);      
          //strcat(lab2,iplab);
       }
	if (argc>=7)
	{
	   startoffset = (long int)atoi(*(argv + 6 ));
	}
       else
       {
          startoffset=0;     // default startoffset
       }

       /* Data header check to get sampling parameters */
       if(checkheader(&X) < 0){
            goto exitend;
       }

//nextstep:

       /* parameter set */
        sfreq=X.sfreq;
        adbit=X.adbit;
        numch=X.numch;
        fsampl=(float)(1000.0*(float)sfreq);   // sampling frequency (Hz)
        tsampl=(float)(1.0/fsampl);            // sampling period (sec)
        bits=((long)fsampl)*adbit*numch;  // # of bits in 1 sec data except header
        bytes=bits/8;                     // # of bytes in 1 sec data except header
        X.fsample=fsampl;
        //sekibun=1;                       // Averaging period for Spectrum (sec)

        smode=1;                         // low resolution mode

        if(smode ==1){ usampl=10000; }
        else {
           usampl=50000;
           if(sfreq == 40){ usampl=40000; }
           if(sfreq == 100){ usampl=100000; }
           if(sfreq == 200){ usampl=200000; }
        }
        imax=(sfreq*1000)/usampl;
        numb=usampl*adbit*numch/8;   // # of bytes in unit # of samples

        printf("imax numb  %d %d\n",imax,numb);    //debug

        nfft=1024;
        if(usampl < 524288){nfft= 524288;}
        if(usampl < 262144){nfft= 262144;}
        if(usampl < 131072){nfft= 131072;}
        if(usampl < 65536){nfft= 65536;}
        if(usampl < 32768){nfft= 32768;}
        if(usampl < 16384){nfft= 16384;}

        ibuf = (unsigned char *)malloc(numb);
        idata = (int *)malloc(4*usampl*sizeof(int));
        cdata = (double *)malloc(2*nfft*sizeof(double));
        spows = (float *)malloc(4*nfft*sizeof(float));
        xdata = (float *)malloc(nfft*sizeof(float));
        ydata = (float *)malloc(nfft*sizeof(float));
        //autocor=(float *)malloc(4*nfft*sizeof(float));

        fact=((float)usampl)*((float)usampl);
        
        /* sampling frequency in text for print */
        if(sfreq < 1000){
           sprintf(labfsampl,"%dkHz\0",sfreq);
        } else {
          sprintf(labfsampl,"%dMHz\0",sfreq/1000);
        }


       /* foward startffset seconds */
       if(fwdNsec(ibuf, startoffset, numb, imax, &X) <0){
              goto exitend;
       }

        for(k=0; k<4*nfft; k++){spows[k]=0.0;}
        for(k=0; k<4; k++){scount[k]=0;}

        printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
        sprintf(ltime1st,"Time(UT) %02d:%02d:%02d\0",X.hh,X.mm,X.ss);
        strcpy(lab2,ltime1st);
        strcat(lab2,"  ");
        strcat(lab2,iplab);
        

       nloop=0;
loop1:
        if(nloop >0) {
           if(checkheader(&X) < 0){ goto exitend; }
            printf("  Time %02d:%02d:%02d   Sec in Day = %d\n",X.hh,X.mm,X.ss,X.seconds);
        }

        for(i=0; i< imax; i++)
        {
             bytesread = fread( ibuf, sizeof( char ), numb, X.stream );
             //printf("bytes read 2 %d\n",bytesread); //debug
             parseADdata_2(ibuf,numb,adbit,idata, &kpos);
             //printf("numb adbit kpos %d %d %d\n",numb,adbit,kpos); //debug
             //printf("ibuf %d %d %d %d\n",ibuf[0],ibuf[1],ibuf[2],ibuf[3]); //debug
             //printf("idata %d %d %d %d\n",idata[0],idata[1],idata[2],idata[3]); //debug
             //for(k=0; k<kpos; k++){printf("k idata %d %d\n",k,idata[k]);} // debug
             for(m=0; m<numch; m++)
             {   
                kaisu=0;
                sum=0.0;
                for(k=0; k<2*nfft; k++){cdata[k]=0.0;} // initialize
                for(k=0; k<kpos/numch; k++)
                {
                   //if(m==0){printf("k idata %d %d\n",k,idata[k*numch+m]);} //debug
                   cdata[2*k]=(double)idata[k*numch+m];        
                   // cdata[2*k+1]=0.0;  
                   sum=sum+cdata[2*k];
                   kaisu++;
                }
             
                dc=sum/(double)kaisu;    // DC component
     
                //printf("m DC = %d %f\n",m,dc);  //debug
                
                for(k=0; k<kpos/numch; k++)
                {
                  //if(m==0){printf("cdata %f",cdata[2*k]);} //debug
                  cdata[2*k]-=dc;
                  //if(m==0){printf(" %f",cdata[2*k]);} //debug
                }
                //hanning(cdata,nfft);  // hanning filter
                //cdft(2*nfft,-1,cdata);
                if(amode ==0) {    // spectrum mode
                    hanning(cdata,usampl);  // hanning filter
                    cdft(2*nfft,1,cdata);
                } else {
                    cdft(2*nfft,-1,cdata);
                }
                for(k=0; k<nfft; k++)
                {
                   pows=(float)((cdata[2*k]*cdata[2*k]+cdata[2*k+1]*cdata[2*k+1])/(double)fact);
                   spows[k*4+m]=spows[k*4+m]+pows;
                }

                scount[m]++;
             }
       }
       nloop++;
       if(nloop < sekibun){ goto loop1;}
       fgetpos(X.stream,&pos);  //debug
	i=fclose( X.stream ) ;
       printf("%d bytes read\n",pos); 

       if(amode > 0) {   // auto correlation function calc
          for(m=0; m<numch; m++){
             for(k=0; k<nfft; k++){
                spows[k*4+m]/=scount[m]; // nomalization
                cdata[2*k]=spows[k*4+m];
                cdata[2*k+1]=0.0;
             }
             if(m==1 || m==3) cdata[0]=1.0;  // to avoid math error
             cdft(2*nfft,1,cdata);
             for(k=0; k<nfft; k++){
               // if(k<10) printf("%d %e\n",k,cdata[2*k]);  // debug
               // if(k>nfft-10) printf("%d %e\n",k,cdata[2*k]);  // debug
                if(k==0) pmax=(float)cdata[0];
                if(cdata[2*k] > pmax) pmax=(float)cdata[2*k];
                spows[k*4+m]=(float)cdata[2*k];
             }
             for(k=0; k<nfft; k++){  // normalize
                spows[k*4+m]/=pmax;
               // if(k<10) printf("   %d %e\n",k,spows[k*4+m]);  // debug
               // if(k>nfft-10) printf("   %d %e\n",k,spows[k*4+m]);  // debug
             }
             
          }
         
       } else {



       	/* convert power to dB  then dBm */       
       	cofst=OFFSET1;       //1 bit sampling full power convert to 0 dB
       	if( adbit==8){cofst=OFFSET8; }    // 8bit sampling offset
       	if( adbit==4){cofst=OFFSET4; }    // 4bit sampling offset
       	if( adbit==2){cofst=OFFSET2; }    // 4bit sampling offset


       	for(m=0; m<numch; m++){
          		for(k=0; k<nfft; k++){
              		spows[k*4+m]/=scount[m]; // nomalization
              		spows[k*4+m]=(float)(10.0*log10(spows[k*4+m]) + cofst); 
             		if(k==0){
               		 pmaxdbm=spows[k*4+m];
               		 pmindbm=spows[k*4+m];
              	} else {
                		if(spows[k*4+m] > pmaxdbm){ pmaxdbm=spows[k*4+m]; }
                		if(spows[k*4+m] < pmindbm){ pmindbm=spows[k*4+m]; }
              	}
           	}
           	pmax=pmaxdbm-cofst;
           	pmin=pmindbm-cofst;
           	printf("CH# %d: Maximum data (dB, dBm) is %f %f\n", m+1,pmax,pmaxdbm);      
        	}

        }

        /* plot spectrum (or auto cor) POSTSCRIPT OUT and XTERM*/

        for(kplot=0; kplot<2; kplot++)
        {
            if(pmode==1 && kplot==1) goto bypass;
            if(pmode==2 && kplot==0) goto bypass;
            //if (cpgopen("?") < 1)
            if(kplot==0) {
               if (cpgopen("/PS") < 1) goto exitend;
            }
            if(kplot==1) {
               if (cpgopen(PG_DISP) < 1) goto exitend;
            }

            cpgsubp(2,2);  // divide plot pannel into 2X2

            for(m=0; m<numch; m++)
            {
                sprintf(lab,"CH#:%1d %1dbit %s sampling, Tavg(sec)=%3d\0",m+1,
                       adbit,labfsampl,sekibun);
                //printf("%s\n",lab);
                for(k=0; k<nfft; k++) 
                {
                   xdata[k]=spows[k*4+m];
                }
                if(amode > 0) {
                     plotautocor(xdata, nfft, sfreq, adbit, lab, lab2, amode);
                } else {
                     plotspe_a(xdata, nfft, sfreq, adbit, lab, lab2, mode);
                }

            }
            /* panel undivided to date write */
            cpgsubp(1,1);
            /* date etc write */
            cpgiden();
            /* close the graphics device */
            cpgclos();
            bypass:;
        }

 exitend: ;
       
       free(ibuf);
       free(cdata);
       free(idata);
       free(spows);
       free(xdata);
       free(ydata);

 exitend2: ;

       return 0;
}
