/*******************************************************************/
/*                                                                 */
/*   Program Name :  timesettk                                     */
/*                                                                 */
/*      Version:  0.00  2001-08-16                                 */
/*                1.00  2002-10-04                                 */
/*                1.10  2003-06-17  add logout                     */
/*                1.11  2003-07-20  add env variable monitor       */
/*                                                                 */
/*      Copyright (c) 2002,2003 T.Kondo/CRL All Right Reserved     */
/*                                                                 */
/*******************************************************************/
/*
/*
     Time Set Program  
        gp@  timesettk year month day hour min sec

          QOOPNXQOPTPOPTbɃZbg

              timesettk 2001 9 20 15 10 15

              ^[̃^C~O͎15bƂȂ
              ZbgƁAbԃ{[h̎\܂̂
              mFB
             
 2003/06/17  Ver 1.10 Oo͂ǉ

*/
#include "chead.h"      // common header for IP-VLBI utilities

#include "libipvlbi.h"  // IP-VLBIp֐Q(by T.Kondo)
#include "libipvlbi_board.h"  // IP-VLBITv[{[hp֐Q(by T.Kondo)

#define K5LOGDIR  "K5LOGDIR"  // environment default for log out directry
#define K5LOGDIRDFLT  "./"    // default log file directory

int y2k(int yyyy, int mm, int dd);

int y2k(int yyyy, int mm, int dd)
{
  int i, sum = 0;
  int month[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30 , 31, 30, 31};

  if(((yyyy % 4 == 0) && (yyyy % 100 != 0)) || (yyyy % 400 == 0))
    month[1] = 29;
  for (i=0; i<mm-1; i++) {
    sum += month[i];
  }
  sum += dd;

  return sum;
}

int main(int argc, char **argv)
{
    char dev[] = "/dev/tds0";  // IP-VLBI BOARD control variable
    int fd_in,rv;
    unsigned int time,day,sec; // foCXǂݍ񂾌݂̎i[
    unsigned int secold, stat;
    int  yyyy, mm, dd;      // Ԑݒ(timeset)ŗp
    int  h, m, s;           // Ԑݒ(timeset)ŗp
    int counter;
    int i;
    char* dflt_logdir=K5LOGDIRDFLT;        // default log file directory
    char logdir[60];               // logfile directory
    char hostname[20];             // host name
    char cbuf[80];
    
    if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           printf("\n");
           exit(0);
         } 
    } 

    if(argc != 7){
        printf("timesettk Ver 1.11 (2003-07-20)\n  Usage: timesettk year month day hour min sec\n");
        exit(0);
    }

    /* parameters get */
    yyyy = atoi(*(argv + 1 ));
    mm   = atoi(*(argv + 2 ));
    dd   = atoi(*(argv + 3 ));
    h    = atoi(*(argv + 4 ));
    m    = atoi(*(argv + 5 ));
    s    = atoi(*(argv + 6 ));

    /* log directory name get from enviroment variable K5LOGDIR if it is set */
    if(getenv(K5LOGDIR) != NULL){
        strcpy(logdir,getenv(K5LOGDIR));
    } else {
        strcpy(logdir,dflt_logdir);
    }

    /* get host name */
    hostget(hostname);

    sprintf(cbuf,"timesettk(1.11) %d %d %d %d %d %d",yyyy,mm,dd,h,m,s);
    if(LogWrite2(logdir,hostname,0,0.0,0,cbuf) == -1) printf("LogWrite2 error!\n");

    fd_in = open(dev, O_RDONLY);  //foCX̃I[v
    if (fd_in == -1) {
        perror("device open() error ");
        sprintf(cbuf,"device open() error");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
    printf("Device open successd No.[%d]\n",fd_in);
    
    /* board status check */
    rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat);
    if (rv == -1) {
        perror("ioctl() status get failed ");
        sprintf(cbuf,"device open() error");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        goto end;
    } else {
        /* satatus get again just in case */
        rv = ioctl(fd_in, TDSIO_GET_STATUS, &stat);
        if (rv == -1) {
           perror("ioctl() status get failed ");
           sprintf(cbuf,"device open() error");
           if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
           goto end;
        }
        //printf("ioctl() status get succeed\n");
        // printf(" status = %X\n",stat);
        if((stat & 1) != 1){
              printf("Warning! 10MHz signal is not supplied. \n");
              sprintf(cbuf,"Warning! 10MHz signal is not supplied.");
              if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        }
        if((stat & 2) != 2){
              printf("Warning! 1PPS signal is not supplied. \n");
              sprintf(cbuf,"Warning! 1PPS signal is not supplied.");
              if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        }
        if((stat & 3) != 3){
              printf("Check above signal connection and try again!\n");
              goto end;
         }
    }
    //foCX玞̎擾
/*
    rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
    if (rv == -1) {
        perror("ioctl() time get failed ");
        return -1;
    }
    printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
    printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
*/

    /* ݒ */

/*
    yyyy = atoi(*(argv + 1 ));
    mm   = atoi(*(argv + 2 ));
    dd   = atoi(*(argv + 3 ));
    h    = atoi(*(argv + 4 ));
    m    = atoi(*(argv + 5 ));
    s    = atoi(*(argv + 6 ));
*/
    printf("%d,%d,%d,%d,%d,%d\n",yyyy,mm,dd,h,m,s);

    day = y2k(yyyy, mm, dd);
    sec = ((h * 60) + m) * 60 + s;
    time = TDS_MAKE_TIME(day,sec);

    printf("Total day  [%5d]  \n",day);
    printf("Sec in day [%5d]  \n",sec);

    //foCXɎ̃Zbg
    rv = ioctl(fd_in, TDSIO_SET_TIME, &time);  
    if (rv == -1) {
        perror("ioctl() time set failed ");
        sprintf(cbuf,"ioctl() time set failed");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
    //printf("time set ok\n");

    //1b̐ݒ
    rv = ioctl(fd_in, TDSIO_SYNC_1PPS);
    if (rv == -1) {
        perror("ioctl() 1pps sync failed ");
        sprintf(cbuf,"ioctl() 1pps sync failed");
        if(LogWrite2(logdir,hostname,0,0.0,3,cbuf) == -1) printf("LogWrite2 error!\n");
        return -1;
    }
    printf("1pps sync ok\n");

    time = 0; //Ôߏ
    secold=90000;
    counter=0;
    close(fd_in);
	
    
    // log out
    sprintf(cbuf,"timesettk (time set OK)");
    if(LogWrite2(logdir,hostname,1,0.0,1,cbuf) == -1) printf("LogWrite2 error!\n");

     //foCX玞̎擾ă`FbN̂߂PObԃ_Ci~bN\
    fd_in = open("/dev/tds0", O_RDONLY);  //foCX̃I[v
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
    do{
        rv = ioctl(fd_in, TDSIO_GET_TIME, &time);
        if (rv == -1) {
            perror("ioctl() time get failed ");
            return -1;
        }
        day=TDS_GET_DAYS(time);
        sec=TDS_GET_SEC(time);
        if (sec!=secold) {
             sec2hms(sec,&h,&m,&s);
             //printf("get count day from device [%5d]\n",TDS_GET_DAYS(time));
             //printf("get count sec from device [%5d]\n",TDS_GET_SEC(time));
             printf("%03d %02d:%02d:%02d\n\033[A",day,h,m,s);
             counter++;
         }
         secold=sec;
    }while(counter <=10);

    printf("\n\n***time setting successed***\n");
end:
    close(fd_in);
    return 0;
}
